/**********************************************************************************************

  Screen Manager Class

  The following class can handle:
	+ Adafruit_ILI9341
	+ Adafruit_STMPE610
    + SMIndicators
    + SMButtons
	+ Strings
	
  This class is used for tracking indicator/button locataions, updating indicator/buttons, and
  identifying which button was last pressed.

  AndyMark Inc.
  Rick Pease
  Nov. 2014

***********************************************************************************************/


#include <ScreenManager.h>
#include <Adafruit_ILI9341.h>
#include <Adafruit_STMPE610.h>


// These are the four touchscreen analog pins
#define YP A2  // must be an analog pin, use "An" notation!
#define XM A3  // must be an analog pin, use "An" notation!
#define YM 5   // can be a digital pin
#define XP 4   // can be a digital pin


Adafruit_ILI9341 A_scrn =  Adafruit_ILI9341(10, 9);

Adafruit_STMPE610 tsa = Adafruit_STMPE610(8);

SMButton b_last; //Button to unclick
//=========================================================
//	CLASS FUNCTIONS
//=========================================================
ScreenManager::ScreenManager(){
	
	current_x = 0;
	current_y = 0;
	
    BACK_COLOR = 0x018B;
	FONT_COLOR = 0xFF9A;

	lastButton = 0;
	
}


void ScreenManager::begin(SMIndicator* i_start, SMIndicator* i_end, SMButton* b_start, SMButton* b_end){
	

	A_scrn.begin();
	if(!tsa.begin()){
		A_scrn.println("ERRRROOOOOOOOOOOOOOOOORRRRRRRRRRRRRRR");
		while(1);	
	}

    A_scrn.fillScreen(BACK_COLOR);
    A_scrn.setRotation(1);
    A_scrn.setTextColor(FONT_COLOR);    
    A_scrn.setTextSize(3);  
	

	
	head = i_start;
	tail = i_end;
	
	b_head = b_start;
	b_tail = b_end;	
	
}


void ScreenManager::setBackColor(int color){
	BACK_COLOR = color;
}

void ScreenManager::setFontColor(int color){
	FONT_COLOR = color;   
	A_scrn.setTextColor(FONT_COLOR);    

}

int ScreenManager::getBackColor(){
	return BACK_COLOR;
}

int ScreenManager::getFontColor(){
	return FONT_COLOR;
}

void ScreenManager::print(String text, SMIndicator &now){

	
	int padd = text.length()*(CHAR_WIDTH-8);
	A_scrn.setTextColor(FONT_COLOR);   
	A_scrn.print(text);	
	current_x += padd;
	now.setXY(current_x,current_y);	
	
	update(now);
}


//
void ScreenManager::println(char* text, SMIndicator &now){
	println(String(text),now);
}

void ScreenManager::println(String text, SMIndicator &now){

	print(text, now);
	current_y += CHAR_HEIGHT;
	current_x = 0;
	A_scrn.setCursor(current_x,current_y);

}
//

//
void ScreenManager::println(String text){
	A_scrn.setTextColor(FONT_COLOR);    
	A_scrn.print(text);	
	current_x = 0;	
	current_y += CHAR_HEIGHT;
	A_scrn.setCursor(current_x,current_y);
}

void ScreenManager::println(char* text){
		println(String(text));
}
//

void ScreenManager::println(void){
	current_x = 0;
	current_y += CHAR_HEIGHT;
	A_scrn.setCursor(current_x,current_y);
}


void ScreenManager::print(String text, SMButton &now){
	
	int padd = text.length()*CHAR_WIDTH;
	A_scrn.setTextColor(FONT_COLOR);    
	A_scrn.print(text);	
	current_x += padd;
	placeButton(now);
	current_x += now.getWidth()+4;

}

void ScreenManager::println(char* text, SMButton &now){
	
	print(String(text),now);	
	current_x = 0;
	current_y += now.getHeight()+4;

}

void ScreenManager::println(SMButton &now){
	println("",now);
}	

void ScreenManager::placeButton(SMButton &now){
	//Box cordinates, text cordinates, text, text length
	now.setXY(current_x,current_y);//Top Left of box
	
	//Shadow
	A_scrn.fillRect(current_x+2,current_y+2,now.getWidth()+15, now.getHeight(),0x52AA);
	
	//Background of button
	A_scrn.fillRect(current_x,current_y,now.getWidth()+15, now.getHeight(),now.getBackColor());
	
	A_scrn.setCursor(current_x+10,current_y+((now.getHeight()-CHAR_HEIGHT+4)/2));
	   
	A_scrn.setTextColor(now.getFontColor());    
	A_scrn.print(now.get());
	//A_scrn.setTextColor(ILI9341_WHITE);   
	now.setValue(now.get());

}

void ScreenManager::clickButton(SMButton *now, int color){
	
	if(b_last.get()!="--"){
		changeButtonBackground(&b_last,b_last.getBackColor());
	}
	b_last = now;
	
	changeButtonBackground(now, 0x6D89);
}

void ScreenManager::changeButtonBackground(SMButton *now, int color){
	//Box cordinates, text cordinates, text, text length
	int b_x = now->getX();
	int b_y = now->getY();
	int b_width = now->getWidth();
	int b_height = now->getHeight();
	
	//Shadow
	A_scrn.fillRect(b_x+2,b_y+2,b_width+15, b_height,0x52AA);
	
	//Background of button
	A_scrn.fillRect(b_x,b_y,b_width+15, b_height, color);
	
	A_scrn.setCursor(b_x+10,b_y+((b_height-CHAR_HEIGHT+4)/2));
	   
	A_scrn.setTextColor(now->getFontColor());    
	A_scrn.print(now->get());
	//A_scrn.setTextColor(ILI9341_WHITE);   
	now->setValue(now->get());
}

void ScreenManager::update(SMIndicator &now){
	if(now.isNew()){
		int x = now.getX();
		int y = now.getY();
		
		A_scrn.fillRect(x,y,now.getWidth(), CHAR_HEIGHT-2,BACK_COLOR);
		A_scrn.setCursor(x,y);
		
		A_scrn.setTextColor(now.getFontColor());    
		A_scrn.print(now.get());
		now.setValue(now.get());
	}
}

void ScreenManager::update(SMIndicator *now){
	
	if(now->isNew()){
		int x = now->getX();
		int y = now->getY();
		
		A_scrn.fillRect(x,y,now->getWidth(), CHAR_HEIGHT-2,BACK_COLOR);
		A_scrn.setCursor(x,y);
		
		A_scrn.setTextColor(now->getFontColor());  
		A_scrn.print(now->get());
		now->setValue(now->get());
	}
}

void ScreenManager::fancyUpdate(void){
	int i = 1;
	SMIndicator* walk = head;
	while((int(walk+i)!=int(tail))&&(i < 20)){	
		update((walk+i));
		i++;
	}
	if(i>=20){
		A_scrn.println("ERROR NEVER FOUND END OF LIST!!");		
	}
	
}

char flag = 255;
String ScreenManager::whichButton(void){
	int i = 1;
	
	String answer = String(flag);
	
	//A_scrn.setCursor(0,16);
	//A_scrn.println("Begin Buffer check ....");
	
	//while(ts.bufferEmpty()) delay(200);
	//A_scrn.print("Check Complete");
	if(!tsa.touched()){
		tsa.writeRegister8(0x4B, 0x01); // Clear the buffer
		tsa.writeRegister8(0x4B, 0x00); // Enable the buffer
		return String(flag);
	}
	TS_Point p = tsa.getPoint();
	//tsa.writeRegister8(STMPE_FIFO_STA, STMPE_FIFO_STA_RESET);
	tsa.writeRegister8(0x4B, 0x01); // Clear the buffer
	tsa.writeRegister8(0x4B, 0x00); // Enable the buffer

		p.x = map(p.x,150,3800,240,0);
		
		p.y = map(p.y,130,4000,0,320);

		answer = String(p.x);
	
	//Check for a button press
	if((p.z > 0) && (p.z< 1000)){	
	//Figure out which button was pressed
	
		//p.x = map(p.x,150,3800,0,240);
		
		//p.y = map(p.y,130,4000,0,320);
		
	    A_scrn.drawPixel(p.y,p.x, 0xF900);
		
		SMButton* walk = b_head;
		
		while((int(walk+i)!=int(b_tail))&&(i < 8)){	
			if((walk+i)->isMatched(p.y, p.x)){
				answer = (walk+i)->get();
				
				clickButton((walk+i),0xF900);
				lastButton = i;
				return String(i);	
			}
			i++;
		}
		
		if(i>=8){
			A_scrn.setCursor(40,40);
			A_scrn.println("ERROR NEVER FOUND END OF LIST!!");		
		}
	}
	//tsa.writeRegister8(0x4B, 0x01); // Clear the buffer
	//tsa.writeRegister8(0x4B, 0x00); // Enable the buffer

	return answer;	
}

int ScreenManager::getLastButton(){
	return lastButton;
}


