/**********************************************************************************************

  RIOduino + TFT Touch Sheild

  The following example uses these hardware:
    + REV Robotics RIOduino          (http://www.andymark.com/SearchResults.asp?Search=rioduino) 
    + Adafruit 2.8" TFT Touch Shield (https://www.adafruit.com/products/1651)

  This example also leverages libraries that were created by 
  the Adafruit which fall under each of their licenses agreements.

  AndyMark Inc.
  Rick Pease
  Nov. 2014

***********************************************************************************************/

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_ILI9341.h>
#include "Adafruit_STMPE610.h"
#include <ScreenManager.h>

//Create a ScreenManager class that will do most of the heavy lifting
ScreenManager screen;


//===============================================
//  INDICATORS
//  Place new indicators below this line
    SMIndicator i_head;//DO NOT ALTER THIS!!!
//===============================================

SMIndicator battery;
SMIndicator rxc;
//SMIndicator radio;
//SMIndicator comms;
//SMIndicator debug;
SMIndicator microns;
//SMIndicator readout;

//===============================================
//  INDICATORS END
//  Place new indicators abouve this line
    SMIndicator i_tail;//DO NOT ALTER THIS!!!
    //Needed to automate updating the indicators
    SMIndicator *i_start = &i_head;
    SMIndicator *i_end = &i_tail;
//===============================================
//===============================================
//  BUTTONS
    SMButton b_head;    
//===============================================

SMButton b_forward;
SMButton b_swerve;
SMButton b_other;
SMButton b_cancel;


//===============================================
//  BUTTONS END
    SMButton b_tail;    
    SMButton *b_start = &b_head;
    SMButton *b_end = &b_tail;
//===============================================




//===============================================
//  Message Flags - Unique ID for each message
//===============================================
#define MSG_BATTY  'B'
#define MSG_RADIO  'R'
#define MSG_COMMS  'C'
#define MSG_MICRN  'M'
#define MSG_DOUT   'c'

String commandToSend = "";

int RXCount = -1; // -1 to enable approximate boot count down

void setup() {  
  delay(500);  //Allows the RIOduino to stabilize before writing to the screen
  
  
  //Initialize buttons with a label
  b_forward   = String("FORWARD");
  b_swerve    = String("CIRCLE");
  b_other     = String("OTHER");
  b_cancel    = String("CANCEL");
  
  
  //Each indicator can have its own Color. Colors are RGB565
  //The following link can be used to convert to the correct formate
  //  http://www.henningkarlsen.com/electronics/calc_rgb565.php


  //Set RX Counter indicator to Orange
  rxc.setFontColor(0xFBE0);
  
  //Set Cancel buttome to to Red
  b_cancel.setFontColor(0xF900);  
  
  //Set Forward to Green
  b_forward.setFontColor(0x07E0);

    
 /* Use the following code to change the background and foreground 
     colors.
     
   screen.setBackColor(0x0000);
   screen.setFontColor(0x80E2);/**/

   //Need to pass the begginning and end of the lists 
   //  of indicators and buttons
   screen.begin(i_start,i_end,b_start,b_end);  //NEEDED


  //This section is used to rearrange the layout of the
  // user interface. 
  
  //====================================================
  // CONSTRUCTING THE USER INTERFACE
  //====================================================  
   /* 1*/screen.println("roboRIO RX: ", rxc);
   /* 2*/screen.println();
   /* 3*/screen.println("Microns : ", microns);
   /* 4*/screen.println("Battery: " , battery);  
   /* 5*/screen.println();
   /* 6*/screen.println("-----------------");
   /* 7*/screen.print(" ", b_forward);
   /* 8*/screen.println(" ", b_swerve);
   /* 9*/screen.print(" ", b_other);
   /*10*/screen.println("  ",b_cancel);
  //====================================================
  // CONSTRUCTING THE USER INTERFACE
  //====================================================  



  //This is used to communicate back to the roboRIO
  Wire.begin(84); 
  Wire.onReceive(handleRX);
  Wire.onRequest(handleTX);
}


char buffer[30];  //Buffer to send data to the roboRIO

String message = "*";  
String lastButton = "";  //Last recorded button pressed
String lastAnalog = "";  //Last recorded analog value 


char fllag = 255;  //Character used to specify no new data

void loop() {

  //If RXCount is in Boot Countdown Mode
  if(RXCount < 0){
     RXCount--; 
     rxc = 35 + (RXCount/10);
  }
  
  
  //Update the indicators and buttons
  screen.fancyUpdate();
  
  //Check if a button has been pressed and store it
  lastButton = screen.whichButton();
  
  //Update the latest analog value  
  lastAnalog = String(analogRead(A0));
  
  //Breathe
  delay(100);
}


//Used to send last recorded data
//commandToSend determines what
//data is returned to the roboRIO
void handleTX(){  
  
  if(commandToSend.indexOf("*")>-1){
    message = commandToSend;
    message.toCharArray(buffer,30);  
    Wire.write(buffer);
    return;
  } 
  
  if(commandToSend.indexOf("Analog0")>-1){
    message = lastAnalog;
    message.toCharArray(buffer,30);  
    Wire.write(buffer);
    return;
  }
  
  if(commandToSend.indexOf("Button")>-1){
    message = String(screen.getLastButton());
    message.toCharArray(buffer,30);  
    Wire.write(buffer);
    return;
  }
  
}

//Handles receiving commands and data from 
//the roboRIO

void handleRX(int numBytes){
  String temp = "";
  int byteNum = 20;
  char text[byteNum+1];
  char blank[byteNum+1];
  int i = 0;
   while(Wire.available()&&(i < (byteNum-1))){
      char c = Wire.read();
        if(c != '\0'){  
          text[i] = c;
          text[i+1] = '\0';
          i++;

        }
   } 
   String data = String(text);
   data = data.substring(1,sizeof(text) - 2);
   
   
   switch (text[0]){
     case MSG_BATTY://'B'
       //New Battery data 
       battery = data;
       break;
       
     case MSG_RADIO: //'R'
       //Example of sending other data 
       //radio = data;
       break;  
       
     case MSG_COMMS://'C'
      // comms = data;
         break;
      case MSG_MICRN://'M'
        microns = data;
       // air = data;
         break; 
      case MSG_DOUT://'c'
        commandToSend = "U";
         break;
     default:
       //Used for when a transmition command is sent
       //      Defined as longer then one character
        commandToSend = data;
       break;
  }

    memset(text,0,(sizeof(text)/sizeof(text[0])));
    
    //Bring RXCount out of Boot Countdown mode
    if(RXCount < 0){
       RXCount = 0; 
    }
    
    //Record another message was received
    rxc = RXCount++; 
   
}

